@@ -35,7 +35,7 @@ void mav_gfxWindowBuffersSwap(void);
 void mav_gfxWindowResGet(int *x, int *y);
 int  mav_gfxWindowEventGet(int *info);
 int  mav_gfxWindowEventPeek(void);
-int  mav_gfxWindowPointerGet(int id, int *x, int *y, int *rx, int *ry);
+int  mav_gfxWindowPointerGet(int id, int *x, int *y, int *rx, int *ry, int *buts);
 void mav_gfxWindowPointerSet(int win, int x, int y);
 int  mav_gfxWindowKeyGet(int key);
 int  mav_gfxWindowFontSet(char *s, int font, int *width);
@@ -124,7 +124,7 @@ void mav_gfxColourSet(MAV_colour col);
 void mav_gfxColourUse(MAV_colour col);
 void mav_gfxMaterialSet(MAV_material mat);
 void mav_gfxMaterialUse(MAV_material mat);
-void mav_gfxTextureSet(MAV_texture tex, MAV_texEnvFn pTexEnv);
+void mav_gfxTextureSet(MAV_texture *tex, MAV_texEnvFn pTexEnv);
 void mav_gfxTextureUse(MAV_texture tex, MAV_texEnvFn pTexEnv);
 void mav_gfxLightingModelSet(MAV_lightingModel lm);
 void mav_gfxLightingModelUse(MAV_lightingModel lm);
@@ -157,6 +157,7 @@ int  mav_gfx3DfxBoardSet(int bd);
 
 extern int mav_opt_trackMatrix;
 extern int mav_opt_bindTextures;
+extern int mav_opt_texComps;
 
 
 
