@@ -153,6 +153,7 @@ typedef struct {
   void *the_data;
   struct MAV_SMS_CLASS *the_class;
   int selectable[MAV_MAX_WIN];
+  void *userdef;
 } MAV_SMS;
 
 typedef int (*MAV_SMSCallbackFn)(MAV_SMS *, void *, void *, void *, void *);
@@ -221,9 +222,10 @@ typedef struct {
   float shine;
 } MAV_material;
 
-typedef void (*MAV_texEnvFn)(void);
+struct MAV_TEXTURE_STRUCT;
+typedef void (*MAV_texEnvFn)(struct MAV_TEXTURE_STRUCT *);
 
-typedef struct {
+typedef struct MAV_TEXTURE_STRUCT {
   int id;
   int defined;
   int width;
@@ -232,6 +234,11 @@ typedef struct {
   char *filename;
   MAV_texEnvFn texEnv;
   int transparent;
+  int mipmapped;
+  int nmaps;
+  int *xsize;
+  int *ysize;
+  unsigned long **mipmap;
 } MAV_texture;
 
 typedef struct {
@@ -429,9 +436,10 @@ void mav_paletteMaterialSet(MAV_palette *p, int index, float ar, float ag, float
 int  mav_paletteTextureSet(MAV_palette *p, int index, char *filename);
 void mav_paletteTextureAlphaSet(MAV_palette *p, int index, float a);
 void mav_paletteTextureColourAlphaSet(MAV_palette *p, int index, int r, int g, int b, float a);
+void mav_paletteTextureMipmappingSet(MAV_palette *p, int index, int v);
 int  mav_paletteTextureSetFromMem(MAV_palette *p, int index, int width, int height, unsigned long *mem);
 int  mav_paletteTextureEnvSet(MAV_palette *p, int index, MAV_texEnvFn fn);
-void mav_texEnvDefault(void);
+void mav_texEnvDefault(MAV_texture *tex);
 void mav_paletteTextureFree(MAV_palette *p, int index);
 void mav_paletteLightSet(MAV_palette *p, int index, float ar, float ag, float ab, float aa, 
 		  float dr, float dg, float db, float da, float sr, float sg, float sb, float sa);
@@ -617,6 +625,7 @@ extern int mav_opt_maxMaterials;
 extern int mav_opt_maxTextures;
 extern int mav_opt_maxFonts;
 extern int mav_opt_maxLights;
+extern int mav_opt_mipmapping;
 extern int mav_opt_objectTableSize;
 
 extern float mav_fps;
