@@ -80,6 +80,7 @@ extern int mav_mouse_x;
 extern int mav_mouse_y;
 extern int mav_mouse_root_x;
 extern int mav_mouse_root_y;
+extern int mav_mouse_button[];
 extern MAV_window *mav_win_mouse;
 extern MAV_vector mav_mouse_pos;
 extern MAV_vector mav_mouse_dir;
@@ -253,7 +254,7 @@ int  mav_exposeDefault(MAV_object *o, MAV_exposeEvent *ee);
 /* Supporting routines */
 
 MAV_line mav_lineFrom2DPoint(MAV_window *w, int x, int y);
-void mav_mouseGet(MAV_window *w, int *x, int *y, int *rx, int *ry);
+void mav_mouseGet(MAV_window *w, int *x, int *y, int *rx, int *ry, int *buts);
 int  mav_keyboardGet(int key);
 extern int mav_drawingMouse;
 void mav_mouseDraw(void);
