@@ -224,27 +224,14 @@ int mav_BBIntersectsLine(MAV_BB bb, MAV_line ln, MAV_objectIntersection *o)
 
 int mav_BBIntersectsBB(MAV_BB bb1, MAV_BB bb2) 
 {
-  MAV_vector c1, c2, v, s;
+  if (bb2.min.x > bb1.max.x) return MAV_FALSE;
+  if (bb2.min.y > bb1.max.y) return MAV_FALSE;
+  if (bb2.min.z > bb1.max.z) return MAV_FALSE;
+  if (bb2.max.x < bb1.min.x) return MAV_FALSE;
+  if (bb2.max.y < bb1.min.y) return MAV_FALSE;
+  if (bb2.max.z < bb1.min.z) return MAV_FALSE;
 
-  /* Find centers of BBs */
-  c1= mav_vectorScalar(mav_vectorAdd(bb1.min, bb1.max), 0.5);
-  c2= mav_vectorScalar(mav_vectorAdd(bb2.min, bb2.max), 0.5);
-
-  /* Find vector between centers */
-  v= mav_vectorSub(c1, c2);
-
-  /* Find combined sizes of BB's */
-  s= mav_vectorAdd(mav_vectorSub(bb1.max, c1), mav_vectorSub(bb2.max, c2));
-
-  /* If distance between centers is less than the combined size, its a hit */
-  if (fabs(v.x)<s.x && fabs(v.y)<s.y && fabs(v.z)<s.z)
-  {
-    return MAV_TRUE;
-  }
-  else
-  {
-    return MAV_FALSE;
-  }
+  return MAV_TRUE;
 }
 
 
