@@ -26,6 +26,12 @@
 */
 
 
+#ifdef WIN32
+#pragma warning( disable : 4244 ) /* '=' : conversion from 'const double ' to 'float ', possible loss of data */
+#pragma warning( disable : 4018 ) /* '<' : signed/unsigned mismatch */
+#pragma warning( disable : 4305 ) /* '=' : truncation from 'const double ' to 'float ' */
+#endif
+
 #include "maverik.h"
 #include "mav_tdm.h"
 
