@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -201,14 +201,17 @@ MAV_object *mavlib_dealWithConeNode(ConeNode *n, MAV_surfaceParams *sp, MAV_matr
 
 /* Routine to generate an indexed face set (a facet) */
 
-MAV_object *mavlib_dealWithIndexedFaceSetNode(IndexedFaceSetNode *n, MAV_surfaceParams *sp, MAV_matrix m)
+MAV_object *mavlib_dealWithIndexedFaceSetNode(IndexedFaceSetNode *n, MAV_surfaceParams *sp, MAV_matrix m, MAV_matrix texTran)
 { 
   int i, j, idx, idxbk, nf=0, nv=0;
   CoordinateNode *pts= NULL; 
+  TextureCoordinateNode *texc= NULL;
   NormalNode *norms= NULL;
   MAV_facet *facet;
-  float v[3];
-  MAV_vector v1, v2;
+  float v[3], vt[2];
+  MAV_vector v1, v2, tvec;
+  MAV_object *o;
+  MAV_BB bb;
   
   /* Get the coordinates node */
   pts= n->getCoordinateNodes();
@@ -216,6 +219,9 @@ MAV_object *mavlib_dealWithIndexedFaceSetNode(IndexedFaceSetNode *n, MAV_surface
   /* Get the normal node */
   norms= n->getNormalNodes();
 
+  /* Get the texture coordinate node */
+  texc= n->getTextureCoordinateNodes();
+
   if (!pts) {
     if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: can not find the indexed face set coordinate node, ignoring\n");
     return NULL;
@@ -229,7 +235,6 @@ MAV_object *mavlib_dealWithIndexedFaceSetNode(IndexedFaceSetNode *n, MAV_surface
 
   /* Misc traps */
   if (n->getColorNodes()) if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: colour not yet supported, ignoring\n");
-  if (n->getTextureNode()) if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: texture not yet supported, ignoring\n");
 
   /* Work with a local copy of the coord indexes for efficiency */
   int coordNum= n->getNCoordIndexes();
@@ -325,6 +330,61 @@ MAV_object *mavlib_dealWithIndexedFaceSetNode(IndexedFaceSetNode *n, MAV_surface
 	idx++;
       }
 
+      /* Store the texture coordinates */
+      if (texc) { // Texture coordinates supplied
+	nv=0;
+	idx= idxbk;
+	while (!(idx==coordNum || coordIdx[idx]==-1)) {
+	
+	  if (n->getNTexCoordIndexes()==0) // Implicit ordering (same as coordIndex)
+	  {	
+	    /* Ensure that the index is valid */
+	    if (coordIdx[idx] >= texc->getNPoints())
+	    {
+	      if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: coordinate index too big, setting to max\n");
+	      texc->getPoint(texc->getNPoints()-1, vt);
+	    }
+	    else
+	    {
+	      texc->getPoint(coordIdx[idx], vt);
+	    }
+	  }
+	  else // Explicit ordering (suppled in texCoordIndex)
+	  {
+	    /* Ensure that the index is valid */
+	    if (n->getTexCoordIndex(idx) >= texc->getNPoints())
+	    {
+	      if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: texture coordinate index too big, setting to max\n");
+	      texc->getPoint(texc->getNPoints()-1, vt);
+	    }
+	    else
+	    {
+	      texc->getPoint(n->getTexCoordIndex(idx), vt);
+	    }
+	  }
+
+	  /* Account for texture transform matrix */
+	  tvec= mav_vectorMult(mav_vectorSet(vt[0], vt[1], 0), texTran);
+	  vt[0]= tvec.x;
+	  vt[1]= tvec.y;
+
+	  /* Account for vertex ordering */
+	  if (n->getCCW())
+	  {
+	    facet->tex[i][nv].s= vt[0];
+	    facet->tex[i][nv].t= vt[1];
+	  }
+	  else
+	  {
+	    facet->tex[i][facet->np[i]-1-nv].s= vt[0];
+	    facet->tex[i][facet->np[i]-1-nv].t= vt[1];
+	  }
+	  
+	  nv++;
+	  idx++;
+	}
+      }
+
       /* Store the normals */
       if (norms)  // Normals supplied
       {
@@ -521,7 +581,80 @@ MAV_object *mavlib_dealWithIndexedFaceSetNode(IndexedFaceSetNode *n, MAV_surface
   delete coordIdx;
   delete perfacenormal;
 
-  return mav_objectNew(mav_class_facet, facet);
+  o= mav_objectNew(mav_class_facet, facet);
+
+  if (!texc && sp->mode>=MAV_TEXTURE) { // Need to automatically generate texture coordinates from BB
+    facet->matrix= MAV_ID_MATRIX; // Local BB
+    if (mav_callbackBBExec(mav_win_current, o, &bb))
+    {
+      float bbax[3], axv[2];
+      int ax[2];
+            
+      /* Calculate size of BB */
+      bbax[0]= bb.max.x-bb.min.x;
+      bbax[1]= bb.max.y-bb.min.y;
+      bbax[2]= bb.max.z-bb.min.z;
+
+      /* Calculate which are the longest 2 dimensions */
+      for (i=0; i<2; i++) { /* s,t */
+	ax[i]= -1;
+	axv[i]= -1.0;
+	for (j=0; j<3; j++) { /* x,y,z */
+	  if (axv[i]<bbax[j]) {
+	    axv[i]=bbax[j];
+	    ax[i]=j;
+	  }
+	}
+	bbax[ax[i]]=-2.0;
+      }
+      
+      /* Calculate texture coordinates */
+      for (i=0; i<facet->npolys; i++) {
+	for (j=0; j<facet->np[i]; j++) {
+
+	  /* S */
+	  switch (ax[0]) {
+	  case 0: // X axis
+	    facet->tex[i][j].s= (facet->vert[i][j].x-bb.min.x)/(bb.max.x-bb.min.x);
+	    break;	    
+	  case 1: // Y axis
+	    facet->tex[i][j].s= (facet->vert[i][j].y-bb.min.y)/(bb.max.y-bb.min.y);
+	    break;	   
+	  case 2: // Z axis
+	    facet->tex[i][j].s= (facet->vert[i][j].z-bb.min.z)/(bb.max.z-bb.min.z);
+	    break;
+	  default:
+	    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: failed to calculate texture coordinates, ignoring\n");
+	    break;
+	  }
+
+	  /* T */
+	  switch (ax[1]) {
+	  case 0: // X axis
+	    facet->tex[i][j].t= (facet->vert[i][j].x-bb.min.x)/(bb.max.x-bb.min.x);
+	    break;	    
+	  case 1: // Y axis
+	    facet->tex[i][j].t= (facet->vert[i][j].y-bb.min.y)/(bb.max.y-bb.min.y);
+	    break;	   
+	  case 2: // Z axis
+	    facet->tex[i][j].t= (facet->vert[i][j].z-bb.min.z)/(bb.max.z-bb.min.z);
+	    break;
+	  default:
+	    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: failed to calculate texture coordinates, ignoring\n");
+	    break;
+	  }	      
+	}
+      }
+    }
+    else
+    {
+      if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: failed to calculate texture coordinates, ignoring\n");
+    }
+
+    facet->matrix= m; // Restore matrix
+  }
+
+  return o;
 }
 
 
@@ -575,23 +708,97 @@ MAV_surfaceParams *mavlib_dealWithMaterialNode(MaterialNode *n)
 
 
 
-/* Routine to deal with an appearance node (i.e. a material definition) */
+/* Routine to build a texture */
 
-MAV_surfaceParams *mavlib_dealWithAppearanceNode(AppearanceNode *n)
-{
-  MaterialNode *mn= n->getMaterialNodes();
-  MAV_surfaceParams *rv= mav_sp_default;
-
-  /* Deal with material node if it exists */
-  if (mn)
+MAV_surfaceParams *mavlib_dealWithImageTextureNode(ImageTextureNode *n)
+{  
+  char *filename;
+  int i=2;
+  
+  if (n->getNUrls()==1)
   {
-    rv= mavlib_dealWithMaterialNode(mn);
+    /* Get filename */
+    filename= n->getUrl(0);
+
+    /* Try to find a matching index */
+    i= mav_paletteTextureIndexMatchGet(mav_palette_default, filename);
+
+    if (i==-1) {
+      /* No joy, find empty texture index to use */
+      i= mav_paletteTextureIndexEmptyGet(mav_palette_default);
+
+      if (i>=0) 
+      {
+	/* Set this texture */
+	if (mav_paletteTextureSet(mav_palette_default, i, filename)==MAV_FALSE) i=2;
+      }
+      else
+      {
+	i=2;
+      }
+    }
   }
   else
   {
-    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: no material node defined in appearance - dont know if this is significant\n");
+    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: %i texture image URLs - dont know how to handle anything other than one\n", n->getNUrls());
   }
 
+  return mav_surfaceParamsNew(MAV_TEXTURE, 0, 0, i);
+}
+
+
+
+/* Routine to deal with texture transforms */
+
+MAV_matrix mavlib_dealWithTextureTransformNode(TextureTransformNode *n)
+{
+  MAV_matrix m, trans, cent, rot, scale, mcent;
+  float c[2], rt, sc[2], tx[2];
+
+  /* Get the transform definition */
+  n->getCenter(c);
+  rt= n->getRotation();
+  n->getScale(sc);
+  n->getTranslation(tx);
+
+  /* Load up the matrices (See VRML97 texture transform node spec) */
+  trans= mav_matrixSet(0,0,0, tx[0], tx[1], 0.0);
+  cent= mav_matrixSet(0,0,0, c[0], c[1], 0.0);
+  rot= mav_matrixQuaternionConvert(mav_quaternionSet(mav_vectorSet(0, 0, 1), MAV_RAD2DEG(rt)));
+  scale= MAV_ID_MATRIX;
+  scale.mat[0][0]= sc[0];
+  scale.mat[1][1]= sc[1];
+  mcent= mav_matrixSet(0,0,0, -c[0], -c[1], 0.0);
+
+  m= mcent;
+  m= mav_matrixMult(m, scale);
+  m= mav_matrixMult(m, rot);
+  m= mav_matrixMult(m, cent);
+  m= mav_matrixMult(m, trans);
+
+  return m;
+}
+
+
+
+/* Routine to deal with an appearance node */
+
+MAV_surfaceParams *mavlib_dealWithAppearanceNode(AppearanceNode *n, MAV_matrix *texTran)
+{
+  MaterialNode *mn= n->getMaterialNodes();
+  ImageTextureNode *tn= n->getImageTextureNodes();
+  TextureTransformNode *ttn= n->getTextureTransformNodes();
+  MAV_surfaceParams *rv= mav_sp_default;
+
+  /* Deal with material node if it exists */
+  if (mn) rv= mavlib_dealWithMaterialNode(mn);
+
+  /* Deal with image texture node if it exists */
+  if (tn) rv= mavlib_dealWithImageTextureNode(tn);
+
+  /* Deal with texture transform node if it exists */
+  if (ttn) *texTran= mavlib_dealWithTextureTransformNode(ttn);
+
   return rv;
 }
 
@@ -605,11 +812,14 @@ MAV_object *mavlib_dealWithShapeNode(ShapeNode *n, MAV_matrix m)
   Node *gn= n->getGeometryNode();
   MAV_surfaceParams *sp= mav_sp_default;
   MAV_object *rv= NULL;
+  MAV_matrix texTran;
+
+  texTran= MAV_ID_MATRIX;
 
   /* Deal with appearance node if it exists */
   if (an)
   {
-    sp= mavlib_dealWithAppearanceNode(an);
+    sp= mavlib_dealWithAppearanceNode(an, &texTran);
   }
   else
   {
@@ -621,7 +831,7 @@ MAV_object *mavlib_dealWithShapeNode(ShapeNode *n, MAV_matrix m)
   {
     if (gn->isIndexedFaceSetNode()) 
     {
-      rv= mavlib_dealWithIndexedFaceSetNode(mavlib_castToIndexedFaceSetNode(gn), sp, m);
+      rv= mavlib_dealWithIndexedFaceSetNode(mavlib_castToIndexedFaceSetNode(gn), sp, m, texTran);
     } 
     else if (gn->isBoxNode()) 
     {
