@@ -28,6 +28,9 @@
 
 #include "maverik.h"
 #include "mavlib_vrad.h"
+#ifdef WIN32
+#include <windows.h>
+#endif
 #include <GL/gl.h>
 
 MAV_class *mavlib_vradElemClass;
