@@ -29,6 +29,17 @@
 #include "mavlib_gfx.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include <math.h>
+
+/* Only OpenGL is applicable for Windows and Macs */
+
+#if defined(WIN32) || defined(macintosh)
+#ifndef MAV_OPENGL
+#define MAV_OPENGL
+#endif
+#endif
+
+
 
 #ifndef MAV_NONE
 #ifdef MAV_IRISGL
@@ -48,6 +59,7 @@ GLuint *mavlib_bindTextureIndex;
 int mav_opt_bindTextures= MAV_TRUE;
 int mav_opt_shareContexts= MAV_TRUE;
 int mav_opt_trackMatrix= MAV_FALSE;
+int mav_opt_texComps= 4;
 
 int mavlib_matrixmode= MAV_MODELVIEW;
 int mavlib_voodoo= MAV_FALSE;
@@ -562,6 +574,18 @@ void mav_gfxTextureEnv1Set(int v)
       glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
       glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
       break;
+    case 3:
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+      break;
+    case 4:
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+      break;
   }
 #endif
 }
@@ -657,14 +681,14 @@ void mav_gfxVisualInfoGet(int *r, int *g, int *b, int *d, int *db, int *ar, int
   *ag=0;
   *ab=0;
 #else
-  glGetIntegerv(GL_RED_BITS, r);
-  glGetIntegerv(GL_GREEN_BITS, g);
-  glGetIntegerv(GL_BLUE_BITS, b);
-  glGetIntegerv(GL_DEPTH_BITS, d);
-  glGetIntegerv(GL_DOUBLEBUFFER, db);
-  glGetIntegerv(GL_ACCUM_RED_BITS, ar);
-  glGetIntegerv(GL_ACCUM_GREEN_BITS, ag);
-  glGetIntegerv(GL_ACCUM_BLUE_BITS, ab);
+  glGetIntegerv(GL_RED_BITS, (GLint *) r);
+  glGetIntegerv(GL_GREEN_BITS, (GLint *) g);
+  glGetIntegerv(GL_BLUE_BITS, (GLint *) b);
+  glGetIntegerv(GL_DEPTH_BITS, (GLint *) d);
+  glGetIntegerv(GL_DOUBLEBUFFER, (GLint *) db);
+  glGetIntegerv(GL_ACCUM_RED_BITS, (GLint *) ar);
+  glGetIntegerv(GL_ACCUM_GREEN_BITS, (GLint *) ag);
+  glGetIntegerv(GL_ACCUM_BLUE_BITS, (GLint *) ab);
 #endif
 #endif
 }
@@ -852,21 +876,67 @@ void mav_gfxMaterialUse(MAV_material mat)
 #endif
 }
 
-void mav_gfxTextureSet(MAV_texture tex, MAV_texEnvFn pTexEnv)
+#define L2 0.301029995
+
+void mav_gfxTextureSet(MAV_texture *tex, MAV_texEnvFn pTexEnv)
 {
 #ifndef MAV_NONE
 #ifdef MAV_IRISGL
   float texprops[]= {TX_MINFILTER, TX_BILINEAR, TX_NULL, TX_MAGFILTER, TX_BILINEAR, TX_NULL, TX_WRAP, TX_REPEAT, TX_NULL};
-  texdef2d(tex.id+1, 4, tex.width, tex.height, tex.mem, 0, texprops);
+  texdef2d(tex->id+1, 4, tex->width, tex->height, tex->mem, 0, texprops);
 #else
+  if (tex->mipmapped) { /* build mipmaps if applicable */
+    int width, height, i;
+
+    width= tex->width;
+    height= tex->height;
+
+    if (width>height)
+    {
+      tex->nmaps= log10(width)/L2;
+    }
+    else
+    {
+      tex->nmaps= log10(height)/L2;
+    }
+    
+    if (tex->nmaps!=0) {
+      tex->xsize= (int *) mav_malloc(tex->nmaps*sizeof(int));
+      tex->ysize= (int *) mav_malloc(tex->nmaps*sizeof(int));
+      tex->mipmap= (unsigned long **) mav_malloc(tex->nmaps*sizeof(unsigned long *));
+    }
+
+    for (i=0; i<tex->nmaps; i++) {
+      if (width>=2) width/=2;
+      if (height>=2) height/=2;
+
+      tex->xsize[i]= width;
+      tex->ysize[i]= height;
+      tex->mipmap[i]= (unsigned long *) mav_malloc(width*height*sizeof(unsigned long));
+#ifdef WIN32
+      gluScaleImage(GL_RGBA, tex->width, tex->height, GL_UNSIGNED_BYTE, tex->mem, width, height, GL_UNSIGNED_BYTE, tex->mipmap[i]);
+#else
+#ifdef MAV_LINUX
+      gluScaleImage(GL_RGBA, tex->width, tex->height, GL_UNSIGNED_BYTE, tex->mem, width, height, GL_UNSIGNED_BYTE, tex->mipmap[i]); /* ABGR_EXT doesnt work for some reason */
+#else
+#ifdef GL_EXT_abgr
+      gluScaleImage(GL_ABGR_EXT, tex->width, tex->height, GL_UNSIGNED_BYTE, tex->mem, width, height, GL_UNSIGNED_BYTE, tex->mipmap[i]);
+#else
+      fprintf(stderr, "Warning: no ABGR texture extension, ignoring.\n");
+#endif
+#endif
+#endif
+    }
+  }
+
 #ifndef MAV_SUNOS4
 #ifdef GL_EXT_texture_object
   if (mav_opt_bindTextures) {
-    glBindTextureEXT(GL_TEXTURE_2D, mavlib_bindTextureIndex[tex.id+1]);
+    glBindTextureEXT(GL_TEXTURE_2D, mavlib_bindTextureIndex[tex->id+1]);
     glEnable(GL_TEXTURE_2D);
 
     mav_opt_bindTextures=MAV_FALSE;
-    mav_gfxTextureUse(tex, pTexEnv);
+    mav_gfxTextureUse(*tex, pTexEnv);
     mav_opt_bindTextures=MAV_TRUE;
 
     mav_surfaceParamsUndefine();
@@ -894,11 +964,14 @@ void mav_gfxTextureUse(MAV_texture tex, MAV_texEnvFn pTexEnv)
 #endif
 #endif
   {
+    int i;
 #ifdef WIN32
-    glTexImage2D(GL_TEXTURE_2D, 0, 4, tex.width, tex.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.mem);
+    glTexImage2D(GL_TEXTURE_2D, 0, mav_opt_texComps, tex.width, tex.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.mem);
+    if (tex.mipmapped) for (i=0; i<tex.nmaps; i++) glTexImage2D(GL_TEXTURE_2D, i+1, mav_opt_texComps, tex.xsize[i], tex.ysize[i], 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.mipmap[i]);
 #else
 #ifdef GL_EXT_abgr
-    glTexImage2D(GL_TEXTURE_2D, 0, 4, tex.width, tex.height, 0, GL_ABGR_EXT, GL_UNSIGNED_BYTE, tex.mem);
+    glTexImage2D(GL_TEXTURE_2D, 0, mav_opt_texComps, tex.width, tex.height, 0, GL_ABGR_EXT, GL_UNSIGNED_BYTE, tex.mem);
+    if (tex.mipmapped) for (i=0; i<tex.nmaps; i++) glTexImage2D(GL_TEXTURE_2D, i+1, mav_opt_texComps, tex.xsize[i], tex.ysize[i], 0, GL_ABGR_EXT, GL_UNSIGNED_BYTE, tex.mipmap[i]);
 #else
     fprintf(stderr, "Warning: no ABGR texture extension, ignoring.\n");
 #endif
@@ -908,11 +981,11 @@ void mav_gfxTextureUse(MAV_texture tex, MAV_texEnvFn pTexEnv)
   /* call correct texture environment function */
   if (tex.texEnv) 
   {
-    (*(tex.texEnv))();
+    (*(tex.texEnv))(&tex);
   }
   else
   {
-    if (pTexEnv) (*pTexEnv)();
+    if (pTexEnv) (*pTexEnv)(&tex);
   }
 #endif
 #endif
@@ -1075,13 +1148,14 @@ void mav_gfxBackfaceCullSet(int i)
 
 int mav_gfxBackfaceCullGet()
 {
-  int rv= MAV_FALSE;
-
 #ifdef MAV_OPENGL
+  GLint rv= MAV_FALSE;
   glGetIntegerv(GL_CULL_FACE, &rv);
+#else
+  int rv= MAV_FALSE;
 #endif
   
-  return rv;
+  return (int) rv;
 }
 
 void mav_gfxDepthTestSet(int i)
