@@ -40,38 +40,53 @@ extern void *mavlib_dlh;
 
 
 
+/* Only OpenGL is applicable for Windows and Macs */
+
+#if defined(WIN32) || defined(macintosh)
+#ifndef MAV_OPENGL
+#define MAV_OPENGL
+#endif
+#endif
+
+
+
 /* X, OpenGL and IrisGL includes as necessary */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
 #include <dlfcn.h>
 #endif
 
 #ifndef MAV_NONE
-#ifndef WIN32
-#include <X11/keysym.h>
-#endif
 #ifdef MAV_IRISGL
+#include <X11/keysym.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <gl/glws.h>
-#else
-#ifdef WIN32
+#elif defined(macintosh)
+#include <agl.h>
+#include <Quickdraw.h>
+#include <Fonts.h>
+#include <Windows.h>
+#include <Events.h>
+#include <MacWindows.h>
+#elif defined(WIN32)
 #include <windows.h>
 #include <GL/gl.h>
 #else
+#include <X11/keysym.h>
 #include <GL/glx.h>
 #endif
+
 #ifdef GL_EXT_texture_object
 extern GLuint *mavlib_bindTextureIndex;
 #endif
-#endif
-#endif
 
-#ifndef MAV_NONE
-#ifdef WIN32
+#ifdef macintosh
+WindowPtr mavlib_rootwin;
+#elif defined(WIN32)
 HINSTANCE mavlib_dpy;
 LONG WINAPI mavlib_winEventHandler(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM lparam);
 #else
@@ -79,15 +94,16 @@ Display *mavlib_dpy;
 Window mavlib_rootwin;
 int mavlib_scrnum;
 #endif
-#endif
 
 
 
 /* Data structure to store window and context of each MAV_window */
 
-#ifndef MAV_NONE
 typedef struct {
-#ifdef WIN32
+#ifdef macintosh
+  WindowPtr win;
+  AGLContext ctx;
+#elif defined(WIN32)
   HDC hdc;
   HWND win;
   HGLRC ctx;
@@ -102,16 +118,18 @@ typedef struct {
 #endif
 #endif
 } MAVLIB_winhand;
-#endif
 
-#ifndef MAV_NONE
 MAVLIB_winhand mavlib_winhand[MAX_WINS];
 int mavlib_currwin;
 #endif
 
+
+
 #ifndef MAV_NONE
 /* Routine to convert from OS's Window into MAV_window id */ 
-#ifdef WIN32
+#ifdef macintosh
+int mavlib_winlookup(WindowPtr w)
+#elif defined(WIN32)
 int mavlib_winlookup(HWND w)
 #else
 int mavlib_winlookup(Window w)
@@ -135,12 +153,12 @@ char *mav_gfxModuleID(void)
 {
 #ifdef MAV_NONE
   return "Graphics (None)";
-#else
-#ifdef MAV_IRISGL
+#elif defined(MAV_IRISGL)
   return "Graphics (IrisGL and X11)";
-#else
-#ifdef WIN32
+#elif defined(WIN32)
   return "Graphics (OpenGL and Windows)";
+#elif defined(macintosh)
+  return "Graphics (OpenGL and MacOS)";
 #else
   if (mavlib_voodoo)
   {
@@ -151,8 +169,6 @@ char *mav_gfxModuleID(void)
     return "Graphics (OpenGL and X11)";
   }
 #endif
-#endif
-#endif
 }
 
 void mavlib_gfxExit(void)
@@ -167,6 +183,16 @@ void mavlib_gfxExit(void)
       DestroyWindow(mavlib_winhand[i].win);
     }
   }
+#elif defined(macintosh)
+  /* shut down windows nicely */
+  for (i=0; i<MAX_WINS; i++) {
+    if (mavlib_winhand[i].win) {
+       aglSetCurrentContext(NULL);
+       aglSetDrawable(mavlib_winhand[i].ctx, NULL);
+       aglDestroyContext(mavlib_winhand[i].ctx);
+       DisposeWindow(mavlib_winhand[i].win);
+    }
+  } /* for */
 #else
   /* turn on auto repeat */
   XAutoRepeatOn(mavlib_dpy);
@@ -195,6 +221,8 @@ int mav_gfxModuleInit()
 #ifdef WIN32
   WNDCLASS wc;
   for (i=0; i<MAX_WINS; i++) mavlib_winhand[i].win= (HWND) NULL;
+#elif defined(macintosh)
+  for (i=0; i<MAX_WINS; i++) mavlib_winhand[i].win= (WindowPtr) NULL;
 #else
   for (i=0; i<MAX_WINS; i++) mavlib_winhand[i].win= (Window) NULL;
 #endif
@@ -204,7 +232,17 @@ int mav_gfxModuleInit()
   mav_moduleNew(mav_gfxModuleID);  
 
 #ifndef MAV_NONE
-#ifdef WIN32
+#ifdef macintosh
+  	/* Initialize Macintosh system */
+	InitGraf(&qd.thePort);
+	InitFonts();
+	InitWindows();
+	InitMenus();
+	TEInit();
+	InitDialogs(NULL);
+	FlushEvents (everyEvent, 0);
+	InitCursor();
+#elif defined(WIN32)
 
   /* The code for Window support is a mixture of tutorials found on web
      and code provided by Joerg Anders. Basically, dont ask me too many
@@ -248,18 +286,20 @@ int mav_gfxModuleInit()
   atexit(mavlib_gfxExit);
 #endif
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
   mavlib_scrnum= DefaultScreen(mavlib_dpy);
   mavlib_rootwin= RootWindow(mavlib_dpy, mavlib_scrnum);
 #endif
 #endif
 
+#ifndef macintosh
   /* use environment variable to look for a Voodoo card */
   voodoo= getenv("MESA_GLX_FX");
   if (voodoo) {
     if (!strcmp(voodoo, "f")) mavlib_voodoo= 1;
     if (!strcmp(voodoo, "fullscreen")) mavlib_voodoo= 1;
   }
+#endif
 
   return 1;
 }
@@ -274,7 +314,12 @@ void mav_gfxWindowResGet(int *xres, int *yres)
   *xres=1280;
   *yres=1024;
 #else
-#ifdef WIN32
+#ifdef macintosh
+  GDHandle gdev = GetMainDevice ();
+  Rect bounds = (*gdev)->gdRect;
+  *yres = bounds.bottom;
+  *xres = bounds.right;
+#elif defined(WIN32)
   *xres= GetSystemMetrics(SM_CXSCREEN);
   *yres= GetSystemMetrics(SM_CYSCREEN);
 #else
@@ -291,7 +336,10 @@ void mav_gfxWindowResGet(int *xres, int *yres)
 void mav_gfxWindowSet(int i)
 {
 #ifndef MAV_NONE
-#ifdef WIN32
+#if defined(macintosh)
+  aglSetDrawable (mavlib_winhand[i].ctx, (CGrafPort *)mavlib_winhand[i].win);
+  aglSetCurrentContext (mavlib_winhand[i].ctx);
+#elif defined(WIN32)
   wglMakeCurrent(mavlib_winhand[i].hdc, mavlib_winhand[i].ctx);
 #else
 #ifdef MAV_IRISGL
@@ -311,7 +359,9 @@ void mav_gfxWindowSet(int i)
 void mav_gfxWindowBuffersSwap(void)
 {
 #ifndef MAV_NONE
-#ifdef WIN32
+#if defined(macintosh)
+  aglSwapBuffers (mavlib_winhand[mavlib_currwin].ctx);
+#elif defined(WIN32)
   SwapBuffers(mavlib_winhand[mavlib_currwin].hdc);
 #else
 #ifdef MAV_IRISGL
@@ -334,7 +384,15 @@ GLuint mavlib_fontBase[MAX_FONTS];
 int mav_gfxWindowFontSet(char *s, int i, int *width)
 {
 #ifdef MAV_OPENGL
-#ifdef WIN32
+#ifdef macintosh
+ int j;
+ mavlib_fontBase[i] = glGenLists((GLuint) 256);
+ if (mavlib_fontBase[i] == 0) return -2;
+ if (!aglUseFont (mavlib_winhand[mavlib_currwin].ctx, 4 /*monaco*/, 1 /*bold*/, 10, 0, 255, mavlib_fontBase[i])) {
+ 	return -1;
+ }
+  for (j=0; j<=255; j++) width[j]= 10;
+#elif defined(WIN32)
   unsigned int first, last;
 
   first=0;
@@ -496,7 +554,7 @@ LONG WINAPI mavlib_winEventHandler(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM l
     mavlib_win32EventInfo[0]= mavlib_winlookup(hwnd);
 
     /* Get pointer position */
-    mav_gfxWindowPointerGet(mavlib_win32EventInfo[0], &mavlib_win32EventInfo[1], &mavlib_win32EventInfo[2], &mavlib_win32EventInfo[3], &mavlib_win32EventInfo[4]);
+    mav_gfxWindowPointerGet(mavlib_win32EventInfo[0], &mavlib_win32EventInfo[1], &mavlib_win32EventInfo[2], &mavlib_win32EventInfo[3], &mavlib_win32EventInfo[4], NULL);
     
     /* Pressed or released */
     mavlib_win32EventInfo[5]=v1;
@@ -598,7 +656,7 @@ LONG WINAPI mavlib_winEventHandler(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM l
     mavlib_win32EventInfo[0]= mavlib_winlookup(hwnd);
 
     /* Get pointer position */
-    mav_gfxWindowPointerGet(mavlib_win32EventInfo[0], &mavlib_win32EventInfo[1], &mavlib_win32EventInfo[2], &mavlib_win32EventInfo[3], &mavlib_win32EventInfo[4]);
+    mav_gfxWindowPointerGet(mavlib_win32EventInfo[0], &mavlib_win32EventInfo[1], &mavlib_win32EventInfo[2], &mavlib_win32EventInfo[3], &mavlib_win32EventInfo[4], NULL);
 
     /* Pressed or released */
     mavlib_win32EventInfo[5]=v2;
@@ -657,6 +715,7 @@ LONG WINAPI mavlib_winEventHandler(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM l
 #endif
 
 
+
 /* Routine to open a window and a OpenGL/IrisGL context */
 
 void mav_gfxWindowOpen(int id, int x, int y, int width, int height, char *nm, int wmplacement, int sb, int qb, int ms, int ab, int *wret, int *hret)
@@ -689,6 +748,58 @@ void mav_gfxWindowOpen(int id, int x, int y, int width, int height, char *nm, in
 
   *wret= width;
   *hret= height;
+
+#elif defined(macintosh)
+  AGLDrawable win;
+  AGLPixelFormat fmt;
+  AGLContext ctx;
+  Rect rect;
+  Str255 title;
+  GLint attrib[20];
+  int i=0;
+  for (i=19; i>=0; i--) {
+  	attrib[i]=AGL_NONE;
+  }
+  i=0;
+  attrib[i]=AGL_RGBA; i++;
+  attrib[i]=AGL_MAXIMUM_POLICY; i++;
+  if (!sb) {
+  	attrib[i] = AGL_DOUBLEBUFFER; i++;
+  }
+  if (ab) {
+  	attrib[i] = AGL_ACCUM_RED_SIZE; i++;
+  	attrib[i] = 4; i++;
+  	attrib[i] = AGL_ACCUM_GREEN_SIZE; i++;
+  	attrib[i] = 4; i++;
+  	attrib[i] = AGL_ACCUM_BLUE_SIZE; i++;
+  	attrib[i] = 4; i++;
+  	/*attrib[i] = AGL_ACCUM_ALPHA_SIZE; i++;
+  	attrib[i] = 4; i++;*/
+  }
+  attrib[i] = AGL_DEPTH_SIZE; i++;
+  attrib[i] = 16; i++;
+  attrib[i] = AGL_NONE;
+  
+  strncpy ((char*)title+1, nm, 255);
+  title[0] = strlen (nm);
+  
+  SetRect (&rect, x, y, width+x, height+y);
+  win = (AGLDrawable) NewCWindow (0L, &rect, title, false, documentProc, (WindowPtr) -1L, false, 0L);
+  if (!win) {
+  	fprintf (stderr, "could not open window\n");
+  	return;
+  }
+  ShowWindow ((GrafPort*) win);
+  HiliteWindow ((GrafPort*) win, true);
+  SetPort ((GrafPort*) win);
+  fmt = aglChoosePixelFormat (NULL, 0, attrib);
+  ctx = aglCreateContext (fmt, NULL);
+  aglSetDrawable (ctx, win);
+  aglSetCurrentContext (ctx);
+  mavlib_winhand[id].win = (GrafPort*)win;
+  mavlib_winhand[id].ctx = ctx;
+
+  *wret = width; *hret = height;
 #else
   XVisualInfo *visinfo=NULL;
   Colormap cmap;
@@ -909,7 +1020,14 @@ void mav_gfxWindowOpen(int id, int x, int y, int width, int height, char *nm, in
 void mav_gfxWindowClose(int id)
 {
 #ifndef MAV_NONE
-#ifdef WIN32
+#if defined(macintosh)
+  if (mavlib_winhand[id].win) {
+     aglSetCurrentContext(NULL);
+     aglSetDrawable(mavlib_winhand[id].ctx, NULL);
+     aglDestroyContext(mavlib_winhand[id].ctx);
+     DisposeWindow(mavlib_winhand[id].win);
+  }
+#elif defined(WIN32)
   wglDeleteContext(mavlib_winhand[id].ctx);
   DestroyWindow(mavlib_winhand[id].win);
   mavlib_winhand[id].win= (HWND) NULL;
@@ -932,7 +1050,33 @@ int mav_gfxWindowEventPeek(void)
   int rv=0;
 
 #ifndef MAV_NONE
-#ifdef WIN32
+#if defined(macintosh)
+  EventRecord event;
+  if (EventAvail (everyEvent, &event)) {
+    switch (event.what) {
+      case mouseDown:
+      case mouseUp:
+        rv=2;
+        break;
+      case autoKey:
+      case keyDown:
+      case keyUp:
+        rv=1;
+        break;
+      case updateEvt:
+        rv=6;
+        break;
+      case activateEvt:
+        rv=5;
+        break;
+      case kHighLevelEvent:
+      default:
+        printf("unknown event %d\n", event.what);
+        rv=-1;
+        break;
+    }
+  }
+#elif defined(WIN32)
   /* TODO - peek event, is this needed? */
 #else
   XEvent event;
@@ -985,6 +1129,9 @@ void mavlib_eventDump(int i, int w)
 {
   printf("Window %i - ", w);
 
+#if defined(macintosh)
+  printf("Event %d\n", i);
+#else
   switch(i) {
 
   case KeyPress:
@@ -1031,6 +1178,7 @@ void mavlib_eventDump(int i, int w)
     printf("Unknown event %i\n", i);
     break;
   }
+#endif
   printf("\n");
 }
 #endif
@@ -1042,17 +1190,211 @@ void mavlib_eventDump(int i, int w)
 */
 
 #ifndef MAV_NONE
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
 int (*mavlib_extraXEventHandler)(XEvent)= NULL;
 #endif
 #endif
 
+#ifdef macintosh
+int mavlib_macLastEvent[20];
+#endif
+
 int mav_gfxWindowEventGet(int *info)
 {
   int rv=0;
 
 #ifndef MAV_NONE
-#ifdef WIN32
+#if defined(macintosh)
+  EventRecord event;
+  
+  if (WaitNextEvent (everyEvent, &event, 100, nil)) {
+    /* first check if the last event was a key press. if so, resend a key release event first. */
+    if (mavlib_macLastEvent[11] == 1 && event.what != autoKey) {
+       mavlib_macLastEvent[5]=1; /* set to key release */
+       mavlib_macLastEvent[11] = 0;
+       mavlib_dealWithKeyboardEvent(mavlib_macLastEvent);
+       mavlib_macLastEvent[11]=0; /* reset */
+    }
+    switch (event.what) {
+      case mouseDown:
+      case mouseUp: {
+ 	    int onwhat;
+		WindowPtr win;
+		  
+        info[0]= mavlib_winlookup(FrontWindow ());
+		onwhat = FindWindow (event.where, &win);
+		/*if (!win) return rv;*/
+		switch (onwhat) { /* test for window handling */
+		    case inSysWindow:
+			   SystemClick(&event, win);/* pass clicks to Desk Accessories */
+			   return rv;
+		    case inMenuBar: {
+		       short menuID, menuEntry;
+		       long choosen = MenuSelect (event.where);
+		       menuID = (choosen>>16)&0xffff;
+		       menuEntry = choosen&0xffff;
+		       if (menuID > 0) {
+		         Str255	DAName;
+		         GetMenuItemText (GetMenuHandle (menuID), menuEntry, DAName);
+		         OpenDeskAcc (DAName);
+		       }
+		       HiliteMenu (0);
+		       return rv;
+		    }
+		    case inContent:
+		       if (info[0] != mavlib_winlookup (win)) {
+		          SelectWindow (win);
+		          return rv;
+		       }
+ 	           break;
+		    case inDrag:
+		       if (info[0] != mavlib_winlookup (win)) {
+		          SelectWindow (win);
+		          return rv;
+		       }
+		       if (info[0]>0) DragWindow (win, event.where, &(*GetGrayRgn())->rgnBBox); return rv;
+		       break;
+		    case inGrow:
+			   if (info[0]>0) {
+			      long newsize;
+			      RgnHandle region;
+			      newsize = GrowWindow (win, event.where, &(*GetGrayRgn())->rgnBBox);
+			      info[1] = newsize&0xffff;
+			      info[2] = (newsize>>16) & 0xffff;
+			      SizeWindow (win, info[1], info[2], true);
+			      aglUpdateContext (mavlib_winhand[info[0]].ctx);
+			      mavlib_dealWithResizeEvent (info);
+			      region = ((WindowPeek)win)->strucRgn;
+			      InvalRgn (region);
+			      return 0; /* must update */
+			   }
+			case inGoAway:
+			   if (info[0]>0) mav_windowDelete (info[0]); return rv;
+		} /* switch onwhat */
+
+	    info[1]= event.where.h - ((info[0]<=0)?0:
+	    		(*(((WindowPeek)mavlib_winhand[info[0]].win)->contRgn))->rgnBBox.left);
+	    info[2]= event.where.v - ((info[0]<=0)?0:
+	    		(*(((WindowPeek)mavlib_winhand[info[0]].win)->contRgn))->rgnBBox.top);
+	    info[3]= event.where.h;
+	    info[4]= event.where.v;
+	
+	    if (event.what==mouseDown) info[5]=0;
+	    if (event.what==mouseUp) info[5]=1;
+	
+		/*printf ("button pos x:%i y:%i rootx:%i rooty:%i\n", info[1],info[2],info[3],info[4]);*/
+	/* set mouse button number */
+	    if (event.modifiers & controlKey) {
+	      info[6]=2; /* middle button when control held */
+	    } else if (event.modifiers & optionKey) {
+	      info[6]=3; /* right button when alt held */
+	    } else {
+	      info[6]=1; /* left button */
+	    }  
+        info[7]=event.modifiers & shiftKey; /* shift modifier */
+        info[8]=event.modifiers & controlKey; /* control modifier */
+        info[9]=event.modifiers & optionKey; /* alt modifier */
+        rv=2;
+        break;
+      }
+      case autoKey:
+      /*case keyUp :*/ /* keyUp events are not always sent to the queue !!! */
+      case keyDown: {
+        char theKey = (event.message & keyCodeMask) >> 8;
+        int theChar = event.message & charCodeMask;
+        info[0]= mavlib_winlookup(FrontWindow ());
+	    info[1]= event.where.h - ((info[0]<=0)?0:
+	    		(*(((WindowPeek)mavlib_winhand[info[0]].win)->contRgn))->rgnBBox.left); /* should be root win */
+	    info[2]= event.where.v - ((info[0]<=0)?0:
+	    		(*(((WindowPeek)mavlib_winhand[info[0]].win)->contRgn))->rgnBBox.top); /* should be root win */
+	    info[3]= event.where.h;
+	    info[4]= event.where.v;
+	
+	    if (event.what==keyDown || event.what==autoKey) {
+	        info[5]=0;
+	    }
+	    if (event.what==keyUp) {
+	    	info[5]=1;
+	    }
+	    /*printf ("key code is %hi  the char is %i or %c\n", theKey, theChar, theChar);*/
+	    
+		switch (theKey) {
+			case 0x7a: theChar = 300; break; /* XK_F1 */
+			case 0x78: theChar = 301; break; /* XK_F2 */
+			case 0x63: theChar = 302; break; /* XK_F4 */
+			case 0x76: theChar = 303; break; /* XK_F5 */
+			case 0x60: theChar = 304; break; /* XK_F5 */
+			case 0x61: theChar = 305; break; /* XK_F6 */
+			case 0x62: theChar = 306; break; /* XK_F7 */
+			case 0x64: theChar = 307; break; /* XK_F8 */
+			case 0x65: theChar = 308; break; /* XK_F9 */
+			case 0x6d: theChar = 309; break; /* XK_F10 */
+			case 0x67: theChar = 310; break; /* XK_F11 */
+			case 0x6f: theChar = 311; break; /* XK_F12 */
+			case 0x7e: theChar = 312; break; /* XK_Up */
+			case 0x7d: theChar = 313; break; /* XK_Down */
+			case 0x7b: theChar = 314; break; /* XK_Left */
+			case 0x7c: theChar = 315; break; /* XK_Right */
+			case 0x74: theChar = 316; break; /* XK_Page_Up */
+			case 0x79: theChar = 317; break; /* XK_Page_Down */
+			case 0x38: theChar = 318; break; /* XK_Shift_L */
+			case 0x3c: theChar = 319; break; /* XK_Shift_R */
+			case 0x3a: theChar = 320; break; /* XK_Alt_L */
+			case 0x3d: theChar = 320; break; /* XK_Alt_R */
+			case 0x37: theChar = 322; break; /* XK_Meta_L  == Apple Key */
+			/* cannot distinguish the meta right from left on the Mac !! */
+			case 0x73: theChar = 324; break; /* XK_Home */
+			case 0x77: theChar = 325; break; /* XK_End */
+			case 0x72: theChar = 326; break; /* XK_Insert */
+			case 0x3b: theChar = 327; break; /* XK_Control_L */
+			case 0x3e: theChar = 328; break; /* XK_Control_R */
+			case 0x39: theChar = 329; break; /* XK_Caps_Lock */
+			default: theKey = 0; break;
+        }
+        info[6]=theChar;
+        info[7]=event.modifiers & shiftKey; /* shift modifier */
+        info[8]=event.modifiers & controlKey; /* control modifier */
+        info[9]=event.modifiers & optionKey; /* alt modifier */
+        { int j;
+          for (j=0; j<20; j++) { /* keep a copy */
+            mavlib_macLastEvent[j] = info[j];
+          }
+          mavlib_macLastEvent[11] = 1; /* save this event identifier */
+        }
+
+        rv=1;
+        break;
+      }
+      case updateEvt:
+        info[0]= mavlib_winlookup((GrafPort*)event.message);
+		info[1]=0;
+        rv=6;
+        break;
+      case activateEvt:
+        info[0]= mavlib_winlookup((GrafPort*)event.message);
+        if (event.modifiers & activeFlag) {
+          info[1]=1;
+        } else {
+          info[1]=0;
+        }
+        rv=5;
+        break;
+       case diskEvt:
+	    if (event.message < 0) {
+	       Point dialogpt = {100, 100};
+		   DIBadMount(dialogpt, event.message);
+	    }
+	    break;
+      case kHighLevelEvent:
+      default:
+        printf("unknown event %i\n", event.what);
+        rv=-1;
+        break;
+    }
+  	/*printf ("event is %i on %i\n", rv, info[0]);*/
+    if (info[0]==-1) rv=-1; /* suppress events for others */
+  }
+#elif defined(WIN32)
   MSG msg;
   int i=0;
 
@@ -1290,7 +1632,7 @@ int mav_gfxWindowEventGet(int *info)
 
 /* Routine to return the position of the mouse in a window and root coords */
 
-int mav_gfxWindowPointerGet(int win, int *x, int *y, int *rx, int *ry)
+int mav_gfxWindowPointerGet(int win, int *x, int *y, int *rx, int *ry, int *buts)
 {
   int rv=1;
 
@@ -1328,6 +1670,19 @@ int mav_gfxWindowPointerGet(int win, int *x, int *y, int *rx, int *ry)
   {
     rv=0;
   }
+#elif defined(macintosh)
+  Point pt;
+  RgnPtr rgn;
+  GetMouse (&pt); /* get it in local coords */
+  if (win > 0) {
+	  rgn = *((WindowPeek)(mavlib_winhand[win].win))->strucRgn;
+	  *x = pt.h; *y = pt.v;
+	  *rx = pt.h+rgn->rgnBBox.left;
+	  *ry = pt.v+rgn->rgnBBox.top;
+      rv = 1;
+  } else {
+      rv = 0;
+  }
 #else
   Window dumroot, dumchild;
   unsigned int btnmask;
@@ -1335,6 +1690,12 @@ int mav_gfxWindowPointerGet(int win, int *x, int *y, int *rx, int *ry)
   if (win>0 && win<MAX_WINS && mavlib_winhand[win].win) 
   {
     XQueryPointer(mavlib_dpy, mavlib_winhand[win].win, &dumroot, &dumchild, rx, ry, x, y, &btnmask);
+
+    if (buts) {
+      buts[0]= !((btnmask>>8) & 0x1); /* left */
+      buts[1]= !((btnmask>>9) & 0x1); /* middle */
+      buts[2]= !((btnmask>>10) & 0x1); /* right */
+    }
   }
   else
   {
@@ -1368,6 +1729,8 @@ void mav_gfxWindowPointerSet(int win, int x, int y)
 
   /* Set cursor pos */
   SetCursorPos(point.x, point.y);
+#elif defined(macintosh)
+  printf ("cannot set mouse pointer \n");
 #else
   XWarpPointer(mavlib_dpy, None, mavlib_winhand[win].win, 0,0,0,0, x, y);
 #endif
@@ -1428,6 +1791,56 @@ int mav_gfxWindowKeyGet(int key)
   {
     return 1; /* Released */
   }
+#elif defined(macintosh)
+  unsigned int whatbyte;
+  unsigned char bitmask;
+  unsigned char theChar=0;
+  unsigned char keys[16];
+  GetKeys ((long*)keys);
+  switch (key) {
+		case 300: theChar =  0x7a; break; /* XK_F1 */
+		case 301: theChar = 0x78; break; /* XK_F2 */
+		case 302: theChar = 0x63; break; /* XK_F4 */
+		case 303: theChar = 0x76; break; /* XK_F5 */
+		case 304: theChar = 0x60; break; /* XK_F5 */
+		case 305: theChar = 0x61; break; /* XK_F6 */
+		case 306: theChar = 0x62; break; /* XK_F7 */
+		case 307: theChar = 0x64; break; /* XK_F8 */
+		case 308: theChar = 0x65; break; /* XK_F9 */
+		case 309: theChar = 0x6d; break; /* XK_F10 */
+		case 310: theChar = 0x67; break; /* XK_F11 */
+		case 311: theChar = 0x6f; break; /* XK_F12 */
+		case 312: theChar = 0x7e; break; /* XK_Up */
+		case 313: theChar = 0x7d; break; /* XK_Down */
+		case 314: theChar = 0x7b; break; /* XK_Left */
+		case 315: theChar = 0x7c; break; /* XK_Right */
+		case 316: theChar = 0x74; break; /* XK_Page_Up */
+		case 317: theChar = 0x79; break; /* XK_Page_Down */
+		case 318: theChar = 0x38; break; /* XK_Shift_L */
+		case 319: theChar = 0x3c; break; /* XK_Shift_R */
+		case 320: theChar = 0x3a; break; /* XK_Alt_L */
+		case 321: theChar = 0x3d; break; /* XK_Alt_R */
+		case 322:
+		case 323: theChar = 0x37; break; /* XK_Meta_L  == Apple Key */
+		case 324: theChar = 0x73; break; /* XK_Home */
+		case 325: theChar = 0x77; break; /* XK_End */
+		case 326: theChar = 0x72; break; /* XK_Insert */
+		case 327: theChar = 0x3b; break; /* XK_Control_L */
+		case 328: theChar = 0x3e; break; /* XK_Control_R */
+		case 329: theChar = 0x39; break; /* XK_Caps_Lock */
+		default: theChar = 0; break;
+  }
+  if (theChar>0) {
+     whatbyte = (int)(theChar/8.0);
+     bitmask = 1<<(theChar%8);
+  /* a special key */
+     return (keys[whatbyte] & bitmask)!=bitmask;
+  } else {
+     whatbyte = (int)(key/8.0);
+     bitmask = 1<<(key%8);
+  /* 7 bit ascii char */
+     return (keys[whatbyte] & bitmask)!=bitmask;
+  }
 #else
   char rv[32], ip[2];
   int keysym=-1, keycode, c, r;
@@ -1498,7 +1911,7 @@ int mav_gfxWindowKeyGet(int key)
 
 void mav_gfx3DfxModeSet(int fullscreen)
 {
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
   typedef unsigned char (*FN)(int);
   FN fn;
   
@@ -1527,7 +1940,7 @@ void mav_gfx3DfxModeSet(int fullscreen)
 int mav_gfx3DfxBoardSet(int bd)
 {
   int rv=0;
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
   typedef unsigned char (*FN)(int);
   FN fn;
   
