@@ -26,5 +26,11 @@
 */
 
 
+#ifdef WIN32
+#pragma warning( disable : 4244 ) /* '=' : conversion from 'const double ' to 'float ', possible loss of data */
+#pragma warning( disable : 4018 ) /* '<' : signed/unsigned mismatch */
+#pragma warning( disable : 4305 ) /* '=' : truncation from 'const double ' to 'float ' */
+#endif
+
 #include "mav_kernel.h"
 #include "mav_gfx.h"
