@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <string.h>
 
 MAV_list *mavlib_frame0_list;
 MAV_list *mavlib_frame1_list;
@@ -353,6 +354,10 @@ void mav_frameBegin(void)
 
   mav_deviceCalc();
 
+  /* Update positions for absolute lights */
+
+  mavlib_lightPosFix();
+
   /* Do frame 2 functions */
 
   mavlib_doingFrame2= MAV_TRUE;
