@@ -31,7 +31,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
 #include <sys/types.h>
 #include <unistd.h>
 #endif
@@ -53,6 +53,7 @@ int mav_opt_maxMaterials=150;
 int mav_opt_maxTextures=150;
 int mav_opt_maxFonts=10;
 int mav_opt_maxLights=5;
+int mav_opt_mipmapping=0;
 
 extern int mavlib_voodoo;
 
@@ -204,6 +205,9 @@ MAV_palette *mav_paletteNew(void)
     rv->texlist[i].id= (mavlib_num_palettes*mav_opt_maxTextures)+i;
     rv->texlist[i].defined=0;
     rv->texlist[i].transparent=0;
+    rv->texlist[i].mipmapped=mav_opt_mipmapping;
+    rv->texlist[i].nmaps=0;
+    rv->texlist[i].mipmap=NULL;
   }
 
   rv->lm.id= mavlib_num_palettes;
@@ -442,6 +446,7 @@ int mavlib_readPPM(char *filename, int *width, int *height, unsigned long **mem)
   float rf, gf, bf;
   int r, g, b, i, j, mv;
   char str[100];
+  unsigned char *c;
   unsigned long *map;
   int bin=0;
 
@@ -466,7 +471,11 @@ int mavlib_readPPM(char *filename, int *width, int *height, unsigned long **mem)
 
   fscanf(file, "%s", str);
   while (str[0]=='#') { /* comment - read to end of line */
+#ifdef macintosh
+    while (str[0]!='\r') fscanf(file, "%c", &str[0]);
+#else
     while (str[0]!=10) fscanf(file, "%c", &str[0]);
+#endif
     fscanf(file, "%s", str);
   }
   
@@ -474,7 +483,11 @@ int mavlib_readPPM(char *filename, int *width, int *height, unsigned long **mem)
   
   fscanf(file, "%s", str);
   while (str[0]=='#') { /* comment - read to end of line */
+#ifdef macintosh
+    while (str[0]!='\r') fscanf(file, "%c", &str[0]);
+#else
     while (str[0]!=10) fscanf(file, "%c", &str[0]);
+#endif
     fscanf(file, "%s", str);
   }
   
@@ -521,10 +534,19 @@ int mavlib_readPPM(char *filename, int *width, int *height, unsigned long **mem)
       gf=((float) g)/mv;
       bf=((float) b)/mv;
 
-#ifdef MAV_LINUX
-      map[i**width+j]=(((int) (rf*255))<<24) + (((int) (gf*255))<<16) + (((int) (bf*255))<<8) + 255;
+      c= (unsigned char *) &map[i**width+j];
+#ifdef WIN32
+      /* Textures have RGBA ordering */
+      c[0]= (unsigned char) (rf*255);
+      c[1]= (unsigned char) (gf*255);
+      c[2]= (unsigned char) (bf*255);
+      c[3]= 255;
 #else
-      map[i**width+j]=(255<<24) + (((int) (bf*255))<<16) + (((int) (gf*255))<<8) + (((int) (rf*255)));
+      /* Textures have ABGR ordering */
+      c[0]= 255;
+      c[1]= (unsigned char) (bf*255);
+      c[2]= (unsigned char) (gf*255);
+      c[3]= (unsigned char) (rf*255);
 #endif
     }
   }
@@ -565,7 +587,7 @@ int mav_paletteTextureSet(MAV_palette *p, int index, char *filename)
     }
     else
     {
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
       char buf[500], f1[500], f2[500];
       int srv;
 
@@ -615,7 +637,7 @@ int mav_paletteTextureSet(MAV_palette *p, int index, char *filename)
 	mav_free(p->texlist[index].mem);
 
 	{
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
 	  char buf[500], f1[500], f2[500];
 	  int srv;
 
@@ -651,7 +673,7 @@ int mav_paletteTextureSet(MAV_palette *p, int index, char *filename)
       p->texlist[index].filename= strdup(filename);
       p->texlist[index].texEnv= NULL;
       
-      mav_gfxTextureSet(p->texlist[index], p->texEnv);
+      mav_gfxTextureSet(&p->texlist[index], p->texEnv);
       mav_surfaceParamsUndefine();
     }
   }
@@ -667,6 +689,7 @@ void mav_paletteTextureAlphaSet(MAV_palette *p, int index, float a)
 {
   MAV_texture *tex;
   int i, v;
+  unsigned char *c;
 
   /* Clamp alpha value and turn into an integer */
   if (a>1.0) a=1.0;
@@ -688,10 +711,13 @@ void mav_paletteTextureAlphaSet(MAV_palette *p, int index, float a)
 
     tex= &p->texlist[index];
     for (i=0; i<tex->width*tex->height; i++) {
-#ifdef MAV_LINUX
-      tex->mem[i]= (tex->mem[i]&0xffffff00) + v;
+      c= (unsigned char *) &tex->mem[i];
+#ifdef WIN32
+      /* Textures have RGBA ordering */
+      c[3]= (unsigned char) v;
 #else
-      tex->mem[i]= (tex->mem[i]&0x00ffffff) + (v<<24);
+      /* Textures have ABGR ordering */
+      c[0]= (unsigned char) v;
 #endif		
     }
 
@@ -705,7 +731,7 @@ void mav_paletteTextureAlphaSet(MAV_palette *p, int index, float a)
       tex->transparent=0;
     }
     
-    mav_gfxTextureSet(p->texlist[index], p->texEnv);
+    mav_gfxTextureSet(&p->texlist[index], p->texEnv);
     mav_surfaceParamsUndefine();
   }
 }
@@ -718,6 +744,7 @@ void mav_paletteTextureColourAlphaSet(MAV_palette *p, int index, int r, int g, i
 {
   MAV_texture *tex;
   int i, v;
+  unsigned char *c;
 
   /* Clamp alpha value and turn into an integer */
   if (a>1.0) a=1.0;
@@ -739,10 +766,13 @@ void mav_paletteTextureColourAlphaSet(MAV_palette *p, int index, int r, int g, i
     
     tex= &p->texlist[index];
     for (i=0; i<tex->width*tex->height; i++) {
-#ifdef MAV_LINUX
-      if (((tex->mem[i]>>24)&0xff)==r && ((tex->mem[i]>>16)&0xff)==g && ((tex->mem[i]>>8)&0xff)==b) tex->mem[i]= (tex->mem[i]&0xffffff00) + v;
+      c= (unsigned char *) &tex->mem[i];
+#ifdef WIN32
+      /* Textures have RGBA ordering */      
+      if (c[0]==r && c[1]==g && c[2]==b) c[3]=(unsigned char) v;
 #else
-      if ((tex->mem[i]&0xff)==r && ((tex->mem[i]>>8)&0xff)==g && ((tex->mem[i]>>16)&0xff)==b) tex->mem[i]= (tex->mem[i]&0x00ffffff) + (v<<24);
+      /* Textures have ABGR ordering */      
+      if (c[3]==r && c[2]==g && c[1]==b) c[0]=(unsigned char) v;
 #endif		
     }
 
@@ -756,13 +786,31 @@ void mav_paletteTextureColourAlphaSet(MAV_palette *p, int index, int r, int g, i
       tex->transparent=0;
     }
     
-    mav_gfxTextureSet(p->texlist[index], p->texEnv);
+    mav_gfxTextureSet(&p->texlist[index], p->texEnv);
     mav_surfaceParamsUndefine();
   }
 }
 
 
 
+/* Routine to define if a texture is mipmapped */
+
+void mav_paletteTextureMipmappingSet(MAV_palette *p, int index, int v)
+{
+  /* Check index is within range */
+  
+  if (index > mav_opt_maxTextures-1) 
+  {
+    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: Texture index %i too big (max %i), ignoring\n", index, mav_opt_maxTextures-1);
+  }
+  else
+  {
+    p->texlist[index].mipmapped= v;
+  }
+}
+
+
+
 /* Routine to define a texture in a given palette (from mem) */
 
 int mav_paletteTextureSetFromMem(MAV_palette *p, int index, int width, int height, unsigned long *mem)
@@ -799,7 +847,7 @@ int mav_paletteTextureSetFromMem(MAV_palette *p, int index, int width, int heigh
       p->texlist[index].height= height;
       p->texlist[index].mem= mem;
       
-      mav_gfxTextureSet(p->texlist[index], p->texEnv);
+      mav_gfxTextureSet(&p->texlist[index], p->texEnv);
       mav_surfaceParamsUndefine();
     }
   }
@@ -838,9 +886,16 @@ int mav_paletteTextureEnvSet(MAV_palette *p, int index, MAV_texEnvFn fn)
 
 /* Default texture environment */
 
-void mav_texEnvDefault(void)
+void mav_texEnvDefault(MAV_texture *tex)
 {
-  mav_gfxTextureEnv1Set(1);
+  if (tex->mipmapped)
+  {
+    mav_gfxTextureEnv1Set(3);
+  }
+  else
+  {
+    mav_gfxTextureEnv1Set(1);
+  }
 }
 
 
@@ -852,6 +907,15 @@ void mav_paletteTextureFree(MAV_palette *p, int index)
   if (p->texlist[index].defined) {
     if (p->texlist[index].filename) mav_free(p->texlist[index].filename);
     if (p->texlist[index].mem) mav_free(p->texlist[index].mem);
+    if (p->texlist[index].mipmapped) {
+      int i;
+      for (i=0; i<p->texlist[index].nmaps; i++) mav_free(p->texlist[index].mipmap[i]);
+      if (p->texlist[index].nmaps!=0) {
+	mav_free(p->texlist[index].xsize);
+	mav_free(p->texlist[index].ysize);
+	mav_free(p->texlist[index].mipmap);
+      }
+    }
     p->texlist[index].defined= MAV_FALSE;
   }
 }
@@ -988,7 +1052,7 @@ int mav_surfaceParamsIsTransparent(MAV_window *win, MAV_surfaceParams *sp)
 
   switch (sp->mode) {
   case MAV_COLOUR:
-    if (win->palette->collist[sp->colour].colour[3]<0.99) rv= MAV_TRUE;
+    if (sp->colour>0 && win->palette->collist[sp->colour].colour[3]<0.99) rv= MAV_TRUE;
     break;
   case MAV_MATERIAL:
     if (win->palette->matlist[sp->material].ambient[3]<0.99 ||
