@@ -365,7 +365,7 @@ int mav_ctorusBB2(MAV_object *obj, MAV_BB *bb)
 
 
 
-/* Routine to solve a quartic taken from XXX  */
+/* Routine to solve a quartic taken from Graphics Gems I  */
 
 /*
  *  Roots3And4.c
@@ -394,7 +394,11 @@ int mav_ctorusBB2(MAV_object *obj, MAV_BB *bb)
  */
 
 #include    <math.h>
+#if defined(WIN32) || defined(macintosh)
+double cbrt(double x) {return pow(x, (double)(1.0/3.0));}
+#else
 double cbrt(double x);
+#endif
 
 /* epsilon surrounding for near zero values */
 
@@ -469,11 +473,8 @@ int mavlib_SolveCubic(double c[4], double s[3])
         }
         else /* one single and one double solution */
         {
-#ifdef WIN32
-            double u = pow(-q, 1.0/3.0);
-#else
             double u = cbrt(-q);
-#endif
+
             s[ 0 ] = 2 * u;
             s[ 1 ] = - u;
             num = 2;
@@ -492,13 +493,9 @@ int mavlib_SolveCubic(double c[4], double s[3])
     else /* one real solution */
     {
         double sqrt_D = sqrt(D);
-#ifdef WIN32
-        double u = pow(sqrt_D - q, 1.0/3.0);
-        double v = - pow(sqrt_D + q, 1.0/3.0);
-#else
         double u = cbrt(sqrt_D - q);
         double v = - cbrt(sqrt_D + q);
-#endif
+
         s[ 0 ] = u + v;
         num = 1;
     }
