@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -708,52 +708,32 @@ int mavlib_signedLinePolygonIntersection(MAV_polygon *apoly, MAV_line ln, MAV_ob
   MAV_vector *new_vert, axis, norm;
   MAV_matrix rot;
   MAV_line rotln;
-  float ca, sa;
+  float ca, sa, ang;
   int i, j, c=MAV_FALSE;
 
 /* Malloc off space to rotate vertices so norm is parrallel to Z axis */
 
   new_vert=mav_malloc(apoly->np*sizeof(MAV_vector));
 
-/* Find rotation and translation so polygon's normal is aligned with Z axis */
+/* Find rotation so polygon's normal is aligned with Z axis */
 
-  if (apoly->norm.x == 0.0 && apoly->norm.y == 0.0) 
+  ca= apoly->norm.z;
+  if (ca>=1.0)
   {
-    axis.x = 1.0;
-    axis.y = 0.0;
-    axis.z = 0.0;
+    rot= MAV_ID_MATRIX;
   }
   else
   {
-    axis.x = -apoly->norm.y;
-    axis.y = apoly->norm.x;
-    axis.z = 0.0;
-  }  
-  
-  axis= mav_vectorNormalize(axis);
+    axis.x= apoly->norm.y;
+    axis.y= -apoly->norm.x;
+    axis.z= 0;
 
-  ca = apoly->norm.z;
-  sa = sqrt(1.0 - ca*ca);
+    ang= MAV_RAD2DEG(acos(ca));
+    sa= mav_vectorMag(axis);
+    if (sa<0.0) ang= 360-ang;
 
-  rot.mat[0][0] = axis.x*axis.x+ca*(1.0-axis.x*axis.x);
-  rot.mat[1][0] = axis.x*axis.y*(1.0-ca)+axis.z*sa;
-  rot.mat[2][0] = axis.z*axis.x*(1.0-ca)-axis.y*sa;
-  rot.mat[3][0] = 0.0;
-
-  rot.mat[0][1] = axis.x*axis.y*(1.0-ca)+axis.z*sa;
-  rot.mat[1][1] = axis.y*axis.y+ca*(1.0-axis.y*axis.y);
-  rot.mat[2][1] = axis.y*axis.z*(1.0-ca)+axis.x*sa;
-  rot.mat[3][1] = 0.0;
-
-  rot.mat[0][2] = axis.z*axis.x*(1.0-ca)+axis.y*sa;
-  rot.mat[1][2] = axis.y*axis.z*(1.0-ca)-axis.x*sa;
-  rot.mat[2][2] = axis.z*axis.z+ca*(1.0-axis.z*axis.z);
-  rot.mat[3][2] = 0.0;
-
-  rot.mat[0][3] = 0.0;
-  rot.mat[1][3] = 0.0;
-  rot.mat[2][3] = 0.0;
-  rot.mat[3][3] = 1.0;
+    rot= mav_matrixQuaternionConvert(mav_quaternionSet(mav_vectorNormalise(axis), ang));
+  }
 
 /* Rotate all the polygon pts to be Z axis aligned */
 
