@@ -30,11 +30,11 @@
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
 #include <dlfcn.h>
 #endif
 #include <string.h>
-#ifndef WIN32
+#if !defined(WIN32) && !defined(macintosh)
 #include <sys/types.h>
 #include <unistd.h>
 #endif
@@ -123,6 +123,7 @@ int mav_mouse_x;
 int mav_mouse_y;
 int mav_mouse_root_x;
 int mav_mouse_root_y;
+int mav_mouse_button[10];
 MAV_window *mav_win_mouse= NULL;
 MAV_vector mav_mouse_pos;
 MAV_vector mav_mouse_dir;
@@ -136,7 +137,11 @@ MAV_viewModifierParams mav_stp_default;
 
 void mavlib_pollWindow(void)
 {  
-  if (mav_win_mouse) mav_mouseGet(mav_win_mouse, &mav_mouse_x, &mav_mouse_y, &mav_mouse_root_x, &mav_mouse_root_y);
+#if defined(macintosh) || defined(WIN32)
+  if (mav_win_mouse) mav_mouseGet(mav_win_mouse, &mav_mouse_x, &mav_mouse_y, &mav_mouse_root_x, &mav_mouse_root_y, NULL);
+#else
+  if (mav_win_mouse) mav_mouseGet(mav_win_mouse, &mav_mouse_x, &mav_mouse_y, &mav_mouse_root_x, &mav_mouse_root_y, mav_mouse_button);
+#endif
 }
 
 
@@ -176,19 +181,27 @@ int mavlib_checkWindowEvents(void)
     rv2= mavlib_dealWithMouseEvent(info);
     break;
   case 3: /* resize event (only deal with the last one) */
+#ifndef macintosh
     while (mav_gfxWindowEventPeek()==rv) mav_gfxWindowEventGet(info);
+#endif
     rv2= mavlib_dealWithResizeEvent(info);    
     break;
   case 4: /* map/unmap event (only deal with the last one) */
+#ifndef macintosh
     while (mav_gfxWindowEventPeek()==rv) mav_gfxWindowEventGet(info);
+#endif
     rv2= mavlib_dealWithMapEvent(info);
     break;
   case 5: /* enter/leave event (only deal with the last one) */
+#ifndef macintosh
     while (mav_gfxWindowEventPeek()==rv) mav_gfxWindowEventGet(info);
+#endif
     rv2= mavlib_dealWithCrossingEvent(info);
     break;
   case 6: /* expose event (only deal with the last one) */
+#ifndef macintosh
     while (mav_gfxWindowEventPeek()==rv) mav_gfxWindowEventGet(info);
+#endif
     rv2= mavlib_dealWithExposeEvent(info);
     break;
   }
@@ -263,8 +276,8 @@ void mavlib_cf1(void)
 
 void mavlib_cf12(void)
 {
-#ifdef WIN32
-  fprintf(stderr, "Warning: dynamic loading of modules is not supported under Windows, ignoring\n"); /* WIN32 */
+#if defined(WIN32) || defined(macintosh)
+  fprintf(stderr, "Warning: dynamic loading of modules is not supported under this OS, ignoring\n"); /* WIN32 */
 #else
   char buf[500], filename[500], initName[100];
   void *dlh;
@@ -549,7 +562,7 @@ int mav_windowsModuleInit(void)
     }
 
     /* find name of executable */
-#ifdef WIN32
+#if defined(WIN32) || defined(macintosh)
     strcpy(buf, "Maverik"); /* WIN32 */
 #else
     sprintf(filename, "/tmp/mavname%i", getpid());
